// line 184 "ExpressionGibbsSampler.jweb"
/*;
Copyright (c) 1998 The Omega Project for Statistical Computing.  
All rights reserved.

*/


// line 166 "ExpressionGibbsSampler.jweb"
   package org.omegahat.Simulation.MCMC.Expressions;


// line 172 "ExpressionGibbsSampler.jweb"
    import org.omegahat.Simulation.MCMC.*;
    import org.omegahat.Simulation.MCMC.Listeners.*;
    import org.omegahat.Simulation.MCMC.Proposals.*;
    import org.omegahat.Simulation.MCMC.Targets.*;

import org.omegahat.Models.Formulae.Expressions.*;
import org.omegahat.Environment.Interpreter.*;
import org.omegahat.Environment.Parser.Parse.*;
import org.omegahat.Environment.Databases.*;
import java.util.Enumeration;

// line 13 "ExpressionGibbsSampler.jweb"
/**;
 * A placeholder for a Gibbs Sampler based on a specified model
 * expression.  It does not yet have the code to compute the correct
 * full conditional distributions that are needed to actually perform
 * the sampling, hence it is marked as <code>abstract</code>.
 */
abstract public class ExpressionGibbsSampler extends ExpressionModel implements GibbsSampler
{


// line 32 "ExpressionGibbsSampler.jweb"
public boolean debug = false;

public MultiLineModelFormula expressions;  // holds the expressions generate values from the full conditional distribions


// line 40 "ExpressionGibbsSampler.jweb"
public boolean debug()
{
  return debug;
}

public boolean debug( boolean debug )
{
  return this.debug = debug;
}



// line 57 "ExpressionGibbsSampler.jweb"
public ExpressionGibbsSampler( MultiLineModelFormula model, 
                                 Database environment ) throws Exception
{
  super( model, environment );

  computeFullConditionalExpressions();
}

public ExpressionGibbsSampler( String   model,
                                 Database environment        ) throws Exception
{
  super( model, environment );

  computeFullConditionalExpressions();
}

public ExpressionGibbsSampler( String[] model,
                                 Database environment ) throws Exception
{
  super( model, environment );

  computeFullConditionalExpressions();
}

// line 87 "ExpressionGibbsSampler.jweb"
/**;
 * Parse the model specification and create expressions that will
 * compute the full conditional distribution of each component. 
 */
abstract protected void computeFullConditionalExpressions();

// line 99 "ExpressionGibbsSampler.jweb"
public Object generate( Object obj ) 
{

  MCMCStateDatabase current = (MCMCStateDatabase) obj;

  // reset search path to contain only the current state and the user-specified environment
  SearchPath s = new SearchPath();
  if(debug) System.out.println("attaching current");
  s.attach( (MCMCStateDatabase) current );
  if(debug) System.out.println("attaching environmente");  
  s.attach( environment );
  searchPath(s);


  // create the new MCMCStateDatabase to hold the proposal
  MCMCStateDatabase retval = new MCMCStateDatabase( "MCMCStateDatabase created by " + this );

  // Iterate over all the variables for which we want to generate a value. */
  for(Enumeration e = model().elements() ; e.hasMoreElements() ;) 
  {
    // get a line from the model
    ModelFormula f = (ModelFormula) e.nextElement();
    if(debug) System.err.println("Trying to handle line : " + f + " ." );

    // get the right hamd side of the expression
    FunctionTransformationExpression ft = (FunctionTransformationExpression) ((List) f.structural()).elementAt(0);

    // now create a new MethodCall using the name of the object and adding the method "generate"
    MethodCall call = new CachingMethodCall( ft.qualifier(), "generate", ft.args() );

    // declare this outside the try so it persists
    Object value;

    if(debug) System.err.println("Trying to evaluate the expression: " + call + "." );
    try
      {
        value = call.eval(this);
      }
    catch( Throwable ex )
      {
        ex.printStackTrace();
        throw new RuntimeException("Failed to evaluate call " + call + ".");
      }

    /* assumes name is of length 1. 
       if not, evaluate it in an assignment.
       new AssignExpression(name, new ConstantExpression(value)).eval(this);
     */
    try
      {
        retval.assign( ((Name) f.response()).asString(), value);  
      }
    catch( Throwable ex )
      {
        ex.printStackTrace();
        throw new RuntimeException("Failed to make assignement to " + f.response() + ".");
      }

  }

  return retval;
} 

// line 27 "ExpressionGibbsSampler.jweb"
}
